#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on 22-may-2023 
esercizo per corso di Python per AIAM 
Pandas e files csv
con calcolo di funzione e grafici

@author: marc
"""
import pandas as pd 
import matplotlib.pyplot as plt


filename="mais2018.csv"
anno=2018          #  anno che sto trattando
tsoglia = 8.0      #  soglia per il calcolo dell'energia accumulata 

# funzione per una stima dell'energia accumulata
def calcolodd(tsoglia,tmin,tmax,tmed):
     if tmax < tsoglia :
         dd=0.0
     else:
         if tmin >= tsoglia:
            dd=tmed-tsoglia 
         else:
            dd=(tmax-tsoglia)**2/(2.0*(tmax-tmin)) 
     return dd 
 

#.... leggo il file csv coi valori .............
# Pandas per fare questo ha funzione con molte opzioni utili
# apre e chiude il file in automatico 
# header mi dice che i nome delle colonne sono nella riga numero 3
# (conta iniziando da zero) e che i dati niziano da li',
# per cui salta le righe numero 0,1,2 

# tmin, tmax, tmed dd ddcumulativa hanno la parte frazionaria
# indicata con la virgola, python vuole il punto,
# il problema si risolve mettendo: decimal=','

df=pd.read_csv(filename,sep=';',
                skipinitialspace=True,skip_blank_lines=True,comment='#',
                header=3,decimal=',')

# per dare un'occhiata a cosa ho letto
#df.info()        # informazioni sul dataframe
#df.head()       # prime righe 

# cambio nomi alle colonne (potevo anche specificarle alla lettura del csv)
df.columns=['mese','giorno','giornoanno','tmin','tmax','tmed','dd',
            'ddcumulativa','col8','col9','col10','col11','das']

# elimino le colonne che non mi servono

df.drop(columns=['col8','col9','col10','col11'],inplace=True)

# uso la colonna das come indice (giorni dalla semina)
df.index=df['das']

# mi creo la data (oggetto datetime.data )
# creo una stringa con la data (per la funzione to_datetime)

df["datastr"]=df["giorno"].astype(str)+"/"+df["mese"].astype(str)+"/"+str(anno)



# converto la stringa in un oggetto datetime
df["data"]=pd.to_datetime(df["datastr"],format="%d/%m/%Y"   )

# ho finito di sistemare il dataframe, controllo come e' fatto
df.info()

# --------- calcolo espressioni con le colonne --------
#------------------------------------------------------

# calcolo della stima dell'energia assorbita
# iterando sul dataframe 
# (calcolo oneroso, ma qui abbiamo solo un migliaio di valori ...)

# mi creo una serie, di float, con gia gli indici giusti
dd1=pd.Series(dtype='float64',index=df['das'])  # creo una Series

# itero sulle righe di df, (esiste itertuples() piu' veloce)
for riga in df.iterrows():  
    nomeserie,serie=riga
    #print(riga)
    dasindex=serie['das']    
    result=calcolodd(tsoglia,serie['tmin'],serie['tmax'],serie['tmed'])
    dd1[dasindex]=result
    
dd2=dd1.round(decimals=1)   # arrotondo, df['dd']  ha solo 1 cifra decimale 

test=dd2!=df['dd']  # Series booleana : True per i risultati diversi
#test=dd1!=df['dd'] # se uso i valori non arrotondati ho differenze

if test.any():
  print("ci sono risultati che non tornano, problemi di approssiazione numerica?")
  # mi faccio un dataframe dei risultati sbagliati
  testdataframe=pd.DataFrame(df['dd'][test])
   # aggiungo la colonne dei risultati calcolati
  testdataframe['dd1']=dd1[test]                     
  print(" i risultati sbagliati sono: ",testdataframe.shape[0])
  print(testdataframe)
    
# ...... altro modo per il calcolo senza iterare   
#  divido il dataframe in parti, per ognuna faccio il calcolo
#  poi  rimetto insieme i risultati
    
    
# selezione la parte di dataframe con (tmax<tsoglia)
# poi quella tmin >= tsoglia, poi l'ultima 

dd3=pd.Series(index=df['das'],dtype='float64') # serie per i risultati

df1=df[df['tmax']<tsoglia] # seleziono con serie booleana 
dd3[df1['das']]=0.0        # i dati di df1 hanno dd=0 
                           # inserisco nella serie dd3 0.0 negli indici di df1
    
df2=df[ (df['tmax'] >= tsoglia) & (df['tmin'] >= tsoglia) ]    
ddcalc2=(df2['tmed']-tsoglia)  # valore di dd per questi  (una serie)
dd3[df2['das']]=ddcalc2  # la aggiungo, coi suoi indici
       
df3=df[ (df['tmax'] >= tsoglia) & (df['tmin'] < tsoglia) ]    
ddcalc3=(df3['tmax']-tsoglia)**2 /(2.0*(df3['tmax']-df3['tmin']))    
dd3[df3['das']]=ddcalc3
    
# aggiungo al dataframe la colonna dei risultati del calcolo
df['dd3']=dd3.round(decimals=1)   

# aggiungo la cumulativa, deve essere uguale alla colonna 'ddcumulativa' : 
    
df['dd3sum'] = df['dd3'].cumsum()   

# ----------- qualche plot per finire ------------------ 
# i dataframe di pandas hanno integrate le  funzioni di matplotlib 
# usando dei default per molti parametri
# si possono mischiare funzioni di matplotlib e di plot per dataframe e series
# 


df.plot(x='das',y='ddcumulativa',title="Stima energia totale assorbita")

plt.clf()  # reset della figura o sovrappone i plot

df['dd'].plot(kind = 'hist',title="Istogramma dell'energia dei giorni ")

plt.clf()

df.plot(x='das',y='dd',title="Energia assorbita neio diversi giorni")


