#!/usr/bin/python3

'''
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
'''

"""
@author: marc

oct-2022

Leggo i files csv della protezione 
civile con i dati sul COVID 
per le regioni
mi faccio grafici e tabelle 

"""

import pandas as pd 
import matplotlib.pyplot as plt
import numpy as np
import datetime 
import sys

# default values

file="dati_regioni.csv" 
annoscelto=2022
regione='Emilia-Romagna'


if(len(sys.argv)>1):
   file=sys.argv[1]  

if(len(sys.argv)>2):  
   annoscelto=int(sys.argv[2])  
   
if(len(sys.argv)>3):  
   regione=sys.argv[3] 
    
   
   
'''
 colonne: 
 0   data     :  datetime64[ns]         *
 1   stato    : "ITA"  
 2   codice_regione : int64 
 3   denominazione_regione   : string
 4   lat            :  float64  
 5   long           :  float64  
            
 6   ricoverati_con_sintomi  :  int64   *    attuali 
 7   terapia_intensiva       :  int64   *    attuali  
 8   totale_ospedalizzati    :  int64   *    attuali 
 9   isolamento_domiciliare  :  int64   *    attuali  
 10  totale_positivi         :  int64   *    attuali 
         
 11  variazione_totale_positivi  : int64       del giorno  
 12  nuovi_positivi              : int64   *   del giorno   
 13  dimessi_guariti             : int64       da inizio 
         
 14  deceduti                    :  int64  *    da inizio 
   
 15  casi_da_sospetto_diagnostico  :  float64   non piu' popolato    
 16  casi_da_screening             :  float64   non piu' popolato  
  
 17  totale_casi                   :  int64      da inizio
       
 18  tamponi                       : int64    * da inizio 
 19  casi_testati                  : int64    * da inizio
       
 20  note                           : float64       
 21  ingressi_terapia_intensiva     : int64    * del giorno
        
 22  note_test                                
 23  note_casi                                 
 24  totale_positivi_test_molecolare      :   int64    da inizio     
 25  totale_positivi_test_antigenico_rapido : int64    da inizio     
 26  tamponi_test_molecolare               :  int64    da inizio     
 27  tamponi_test_antigenico_rapido        :  int64    da inizio     
 28  codice nuts 1
 28  codice nuts 2 
'''  

# ......  leggo il csv 

df=pd.read_csv(file,parse_dates=[0],sep=',')


df.info()

# .....  alcuni controlli: 
    
#   --> totale_ospedalizzati=ricoverati_con_sintomi + erapia_intensiva

# da relazioni fra colonne creo una Series booleana
okrow=df['ricoverati_con_sintomi']+df['terapia_intensiva']==df['totale_ospedalizzati']

print(" Ho errori nel numero ospedalizzati ?",okrow.size != df.shape[0])
#   df[okrow]  # seleziona le righe senza errori
#   df['nomecolonna'] invece e' una series con una colonna

# altro modo di fare il test: 
wrongrow=df['ricoverati_con_sintomi']+df['terapia_intensiva']!=df['totale_ospedalizzati']
print('wrong rows per somma tot ospedalizzati ? ',wrongrow.empty) # tutti False e' empty


# .... selezione un anno (dt accede elle proprieta' del  DatetimeIndex)

df=df[df['data'].dt.year==annoscelto]

# ... selezione di una regione 

df=df[df['denominazione_regione']==regione]


# --> variaz. tot positivi == tot positivi - quelli del giorno prima

wrong=df.iloc[:,10].diff()!=df.iloc[:,11]

# la prima riga non ha valore precedente non la conto 
# any e' vero se c'e' qualche elemento True

print("wrong rows per variazione_totale_positivi? :",wrong.iloc[5:].any() )


# ..... seleziono le colonne che mi interessano, aggiustando i nomi 

 
dff=df.take([0,6,7,9,10,14,17,18,19,21],axis=1)

# cambio nomi alle colonne
#               0      6  tot         7 tot      9  tot    10    14         17       18        19       21

dff.columns=['data','subintensiva','intensiva','isolati','totpos','totdead','totcasi','tottamp','tottest','ingrint']

# i morti del giorno 

dead=dff['totdead'].diff()

# aggiungo la colonna dei morti giornalieri

dff['morti']=dead

#  ........... plot andamento generale ................

#  totale positivi attuali

ax0=dff.plot(x='data',y='totpos',title="Positivi attuali")
plt.show()

#  totali isolati, intensive, subintensive

x='data'
y=['isolati','subintensiva','intensiva']
t=['Isolati','Subintensiva','Intensiva'] # titoli grafici, parametro: title=t

ax=dff.plot(x=x,y=y,subplots=True,layout=(3,1),sharex=True)
plt.show()

# nuovi positivi, ingressi intensiva, morti

# i morti del giorno 

dead=dff['totdead'].diff()

# aggiungo la colonna dei morti giornalieri

dff['morti']=dead 


# ... plot nuovi positivi, ingressi intensiva, morti 

dff2=pd.DataFrame() 

dff2['nuovi casi']=df['nuovi_positivi']
dff2['nuove intensive']=df['ingressi_terapia_intensiva']
dff2['morti']=dff['morti']
dff2.index=df['data']
dff2['data']=dff2.index

y2=['nuovi casi','nuove intensive','morti']

ax2=dff2.plot(y=y2,subplots=True,layout=(3,1),sharex=True)
plt.show()

# ... plot rapporto nuovi casi / nuove intensive 

dff2['rapporto']=dff2['nuove intensive']/dff2['nuovi casi']

#ax3=dff2.plot(y=dff2['rapporto'],title="Rapporto nuove intensive/nuovi casi")
ax3=dff2.plot(y=['rapporto'],title="Rapporto nuove intensive/nuovi casi")
plt.show()

# ..... inserisco il giorno della settimana 

dff2['weekday']=dff2['data'].dt.weekday  # 0 -> lunedi

dff2['day']=dff2['data'].dt.day_name() # da versione 1, prima era weekday_name

dff2['week']=dff2['data'].dt.week # week deprecated c'e' isocalendar().week

# ........... uso funzione pivot per i valori per weekday

casi=dff2.pivot(index='week',columns='weekday',values='nuovi casi')

intensive=dff2.pivot(index='week',columns='weekday',values='nuove intensive')

morti=dff2.pivot(index='week',columns='weekday',values='morti')

print('\nNuovi casi ')
print(casi)

print('\nNuove ntensive')
print(intensive)

print('\nMorti odierni')
print(morti)

#  ...variazione percentueale nuovi casi

perc=(casi.pct_change())*100.0

print("\nVariazione percentuale nuovi casi")
print(perc)      














